%% Computation of multiscale entropy from AR parameters
%  INPUT:  AR parameters Am, Su
%              scale factor tau
%              upsampling factor s, such that cutoff f_tau=s/(2*tau)

function [MSEdws, MSEdws_n, MSEavg, Edws,Eavg, B0, Bm] = ar2mse(Am,Su,tau,s,ncoeff)

%%% upsampling
Ams=[];
for iv=1:length(Am)
    Ams=[Ams zeros(1,floor(s)-1) Am(iv)];
end
if isempty(Ams), Ams=0; end % to handle the case of white noise (AR process of order 0)      

%%% stability check
eps=length(Ams); M=1;
Es=eye(M*eps);AAs=[Ams;Es(1:end-M,:)];lambda=eig(AAs);lambdamax=max(abs(lambda));
if lambdamax>=1% stability check
    error('The simulated VAR process is not stable');
end;

%%% FILTERING
if tau==1
    q=0; b=1;
else
    q=ncoeff; % number of filter coeffs
    ft=1/(2*tau); %cutoff frequency
    Wn=2*ft; %normalized cutoff frequency (fNyquist=1)
    b=fir1(q,Wn,'noscale'); %Hamming window, linear phase (symmetry of b coeffs)
end
B0=b(1);
Bm=b(2:end);

% ISS parameters
[A,C,K,V] = varma2iss(Ams,Bm,Su,B0); % max(abs(eig(A-K*C)))
ret_flt = iss_PV(A,C,K,V);
Eavg=0.5*log(2*pi*exp(1)*ret_flt.Sigmaj);
MSEavg=0.5*log(2*pi*exp(1)*ret_flt.Sigmaj_j);

%%% DOWNSAMPLING
[Ad,Kd,Vd] = iss_ds(A,C,K,V,tau);
Cd=C; %Rtau=R;

ret_dws = iss_PV(Ad,Cd,Kd,Vd); % partial variances
Edws=0.5*log(2*pi*exp(1)*ret_dws.Sigmaj);
MSEdws=0.5*log(2*pi*exp(1)*ret_dws.Sigmaj_j);

Edws_n=0.5*log(2*pi*exp(1)); 
MSEdws_n=MSEdws-Edws+Edws_n; % MSE for normalized process

end


