%% Test of MSE analysis (linear and refined MSE) on an example of cardiovascular time series 
clear;close all;clc;

%%% parameters
whichseries=3; % 1)SAP 2)DAP 3)RR 4)RESP

pmax=12; %maximum  order for model order selection (Bayesian Informatio criterion)
m_emb=2; % m refined MSE estimator (embedding dimension)
rfact=0.2; % r of SampEn estimator, expressed as fraction of signal std.dev.
removetrend='n'; % 'y' to detrend

ncoeff_FIR=48;% set the number of coeff for FIR lowpass filter for linear MSE
ncoeff_BUT=6; % % set the number of coeff for Butterworth lowpass filter for refined MSE

% vector of cutoff frequencies
s_vect   = [1 8 4 7  3 5 1 8  2 7  3  1 1 3  1  1];
tau_vect = [1 9 5 10 5 9 2 18 5 20 10 4 5 20 10 20];
f_taus=s_vect./(2*tau_vect);

%% Analysis 
nscales=length(f_taus);

data=load('example.txt');
Yo=data(:,whichseries);
Yo_m=mean(Yo); Yo_sd=std(Yo);

if removetrend=='y'
    Ytmp=detrend(Yo);
    Y=(Ytmp-mean(Ytmp))';
else
    Y=(Yo-mean(Yo))';
end
    
% model identification
[pottaic,pottbic,aic,bic] = mos_idMVAR(Y,pmax,0); pest=pottbic;
[eAm,eSu,Yp,Up]=idMVAR(Y,pest,0);

% multiscale analysis   
for is=1:nscales
    clc;disp(['scale no. ' int2str(is)]);
    
    s=round(s_vect(is));
    tau=round(tau_vect(is));
    
    %%% LINEAR MSE
    [tmp1, tmp2]=ar2mse(eAm,eSu,tau,s,ncoeff_FIR);
    eMSEss(is)=tmp2;
        

    %%% REFINED MSE
    % filter data
    ft=s/(2*tau); %normalized cutoff frequency
    Wn=2*ft; %normalized cutoff frequency (fNyquist=1)
    if Wn<1
        [b,a]=butter(ncoeff_BUT,Wn); %SHOULD DO THIS TO APPLY VALENCIA RMSE! 
        Ytilda=filtfilt(b,a,Y')'; Ntilda=length(Ytilda);
    else
        Ytilda=Y; Ntilda=length(Ytilda);
    end
    % downsample data
    Ybar=resample(Ytilda',round(s),tau)';
    Ybar=(Ybar-mean(Ybar))./std(Ybar); % normalization
    VL=its_SetLag(m_emb,1,1,0);
    ObsMat=its_buildvectors(Ybar',1,VL);
    Hy_y1=its_CEker(ObsMat,rfact*std(Ybar)); % alla Valencia 
    eMSEref(is)=Hy_y1;
    
end
    
%% plot results  
set(0,'defaultAxesFontSize', 12);
set(0,'defaultTextFontSize', 12);
col1=[255 128 0]/255; %orange
col2=[64 128 0]/255; %green
col3=[0 128 255]/255; % lighblue
colori=[col1; col3];
stringaseries{1}='Systolic Pressure';
stringaseries{2}='Diastolic Pressure';
stringaseries{3}='Heart Period';
stringaseries{4}='Respiration';

figure(1); clf;
subplot(2,3,[1 2]);plot(Yo,'k');
xlabel('n, beats'); ylabel('x(n)'); title([stringaseries{whichseries}, ', original']);
subplot(2,3,[4 5]);plot(Y,'k');
xlabel('n, beats'); ylabel('x(n)'); title([stringaseries{whichseries}, ', zero-mean detrended']);

dx=0.005;
subplot(2,3,[3 6]);
plot(f_taus-dx,eMSEss,'s-','markerfacecolor',colori(1,:),'Color',colori(1,:),'Linewidth',1.5);
hold on;
plot(f_taus+dx,eMSEref,'o-','markerfacecolor',colori(2,:),'Color',colori(2,:),'Linewidth',1.5);
ylabel('\it{C_x}\rm{(\tau)}');
xlabel('\it{f}\rm{_{\tau}}');
xlim([min(f_taus)-2*dx max(f_taus)+2*dx]);
title('Multiscale Entropy');
set(gca,'XDir','reverse');
hleg=legend('LMSE','RMSE','location','north');

set(gcf, 'Position', [100 100 1000 600]);
set(gcf, 'Color', 'w');





