%% Multiscale Entropy Gaussian -test for linear VAR processes
%%% TEST theoretical vs estimated values (LIN, SS, RMSE)
%%% analysis with fixed steps of f_tau
clear; clc; close all; 

%% Parameters
N=300; % length of estimates
numsimu=100;

%%% AR parameters -see 4 configurations decided to be tested
M=1; 
ro0=0; %r0=0.65 (fr0=0)
ro1=0.8; fr1=0.1;
ro2=0; fr2=0.2; figlab='ab'; % 1st configuration - figs a,b
% ro2=0.9; fr2=0.2; figlab='cd'; % 2nd configuration - figs c,d
Su=1; 
        
%%% par of estimators
pcrit='bic'; % 'aic', 'bic', or number for fixed model order
pmax=12; %look for BIC
m_emb=2; % m of Lin and MSEref estimators (embedding dimension)
rfact=0.2; % r of SampEn estimator, expressed as fraction of signal std.dev.
lo=10; hi=90; %percentiles (for plots)

ncoeff_FIR=48;
ncoeff_BUT=6; % 48 for FIR, 6 for Butterworth

%%% scales to be explored
s_vect   = [1 8 4 7  3 5 1 8  2 7  3  1 1 3  1  1];
tau_vect = [1 9 5 10 5 9 2 18 5 20 10 4 5 20 10 20];
f_taus=s_vect./(2*tau_vect);

massimoMSEref=100;% max value for MSE refined (substitute inf)

esporta='n';


%% generate AR polynomial
z0=ro0;
z1a=ro1*(cos(2*pi*fr1)+1i*sin(2*pi*fr1)); z1b=ro1*(cos(2*pi*fr1)-1i*sin(2*pi*fr1));
z2a=ro2*(cos(2*pi*fr2)+1i*sin(2*pi*fr2)); z2b=ro2*(cos(2*pi*fr2)-1i*sin(2*pi*fr2));
z=[z0; z1a; z1b; z2a; z2b];
Apol=poly(z); Apol(Apol==0)=[];
Am=-Apol(2:length(Apol));
p=length(Am);



%% MSE Analysis
nscales=length(f_taus);
MSEth=nan*ones(nscales,1); %theoretical
eMSElin=nan*ones(nscales,numsimu); %linear estimate
eMSEss=nan*ones(nscales,numsimu); % state space estimate
eMSEref=nan*ones(nscales,numsimu); % refined MSE estimate
for is=1:nscales
    s=round(s_vect(is));
    tau=round(tau_vect(is));
    
    %%% theoretical MSE
    clc; disp(['s=' int2str(s) ', theoretical...']);
    [tmp1, tmp2]=ar2mse(Am,Su,tau,s,ncoeff_FIR);
    MSEth(is)=tmp2;
    
    %%% ESTIMATES
    for in=1:numsimu
        clc;disp(['scale no. ' int2str(is) ', estimate ' int2str(in) ' of ' int2str(numsimu) '...']);
        
        U=InstModelfilter(N,Su,'StrictlyCausal'); % gaussian innovations with covariance Su
        Y=MVARfilter(Am,U);
        
        %%% new approach: MSEgauss
        if pcrit(1)=='a' || pcrit(1)=='b'  %model order selection
            [pottaic,pottbic,aic,bic] = mos_idMVAR(Y,pmax,0);
            if pcrit(1)=='a', pest=pottaic; else pest=pottbic; end
        else
            pest=pcrit;
        end
        % model identification
        [eAm,eSu,Yp,Up]=idMVAR(Y,pest,0);
        [tmp1, tmp2]=ar2mse(eAm,eSu,tau,s,ncoeff_FIR);
        eMSEss(is,in)=tmp2;  
        
        %%% Refined MSE estimator (not state space)
        % filter data
        ft=s/(2*tau); %cutoff frequency
        Wn=2*ft; %normalized cutoff frequency (fNyquist=1)
        if Wn<1
            %%%% FIR
            %b=fir1(ncoeff,Wn,'noscale'); %Hamming window, linear phase (symmetry of b coeffs)
            %Ytilda=filter(b,1,Y')';
            %Ytmp=Ytilda(floor(ncoeff/2)+1:end); %plot(Y(1:end-ceil(ncoeff/2))); hold on; plot(Ytilda,'r');
            %Ytilda=Ytmp; Ntilda=length(Ytilda);
            %%%% BUTTERWORTH
            [b,a]=butter(ncoeff_BUT,Wn); %SHOULD DO THIS TO APPLY VALENCIA RMSE! 
            Ytilda=filtfilt(b,a,Y')'; Ntilda=length(Ytilda);
            %subplot(2,1,1);plot(Ytilda);subplot(2,1,2);plot(Ytilda2);
        else
            Ytilda=Y; Ntilda=length(Ytilda);
        end
        % downsample data
        Ybar=resample(Ytilda',round(s),tau)'; %plot(Ytilda); hold on; plot(Ybar,'r');
        Ybar=(Ybar-mean(Ybar))./std(Ybar); % normalizzo!
        %%% REFINED MSE
        VL=its_SetLag(m_emb,1,1,0);
        ObsMat=its_buildvectors(Ybar',1,VL);        
        Hy_y1=its_CEker(ObsMat,rfact*std(Ybar)); % alla Valencia 
        %Hy_y1=its_CEker(ObsMat,rfact*std(Y)); % alla Costa
        eMSEref(is,in)=Hy_y1;

    end
end

%% figure for paper
set(0,'defaultAxesFontSize', 18);
set(0,'defaultTextFontSize', 18);
set(0,'defaultTextFontName', 'Times');
set(0,'defaultAxesFontName', 'Times');
eMSEref(eMSEref==Inf)=massimoMSEref;

%eMSEss_m=median(eMSEss,2);
eMSEss_r=prctile(eMSEss,[lo 50 hi],2);
eMSEref_r=prctile(eMSEref,[lo 50 hi],2);
eMSEref_r(end-1:end,:)=[]; %truncate last 2 values because of unstable estimate
f_taus_r=f_taus; f_taus_r(end-1:end)=[]; %truncate last 2 values f-axis because of unstable estimate

yminn_ss=0.2; ymaxx_ss=2;
yminn_ref=0.2; ymaxx_ref=3;

colth=[128 0 0]/255; % color of theoretical value
colm=[86 86 158]/255; % color of median value
colsh=[189 185 219]/255; %color of shades

figure(1); clf
subplot(1,2,1);
h1=area(f_taus,[eMSEss_r(:,1) eMSEss_r(:,2)-eMSEss_r(:,1) eMSEss_r(:,3)-eMSEss_r(:,2)]); hold on
set(h1(1),'FaceColor','w'); set(h1(2),'FaceColor',colsh); set(h1(3),'FaceColor',colsh);
set(h1(1),'EdgeColor','k'); set(h1(2),'EdgeColor',colsh); set(h1(3),'EdgeColor',colsh); set(h1(3),'LineWidth',1.5);
plot(f_taus,eMSEss_r(:,2),'Color',colm,'LineWidth',1.5);
plot(f_taus,MSEth,'Color',colth,'LineWidth',1.5);
hleg=legend('','est., dispersion','true','est., median','true','location','NorthWest');
set(hleg,'FontSize',18); %set(hleg,'visible','off');
legend boxoff;
title(['(' figlab(1) ') LMSE'],'FontSize',22);
% text(0.45,1.8,['\rho_1=' num2str(ro1) ', f_1=' num2str(fr1) ', \rho_2=' num2str(ro2) ', f_2=' num2str(fr2)])
ylabel('\it{C_x}\rm{(\tau)}','FontSize',22);
xlabel('\it{f}\rm{_{\tau}}','FontSize',22); xlim([min(f_taus) max(f_taus)]);
% xlim([0 max(f_taus)+0.01]);
ylim([yminn_ss ymaxx_ss]);
set(gca,'XDir','reverse');
set(gca,'LineWidth',1);
set(gca,'XTick',0:0.1:0.5);
set(gca,'XTickLabel',{'0','0.1','0.2','0.3','0.4','0.5'});

subplot(1,2,2);
h1=area(f_taus_r,[eMSEref_r(:,1) eMSEref_r(:,2)-eMSEref_r(:,1) eMSEref_r(:,3)-eMSEref_r(:,2)]); hold on
set(h1(1),'FaceColor','w'); set(h1(2),'FaceColor',colsh); set(h1(3),'FaceColor',colsh);
set(h1(1),'EdgeColor','k'); set(h1(2),'EdgeColor',colsh); set(h1(3),'EdgeColor',colsh); set(h1(3),'LineWidth',1.5);
plot(f_taus_r,eMSEref_r(:,2),'Color',colm,'LineWidth',1.5);
plot(f_taus,MSEth,'Color',colth,'LineWidth',1.5);
% hleg=legend('','est., dispersion','true','est., median','true','location','NorthWest');
% set(hleg,'FontSize',16); %set(hleg,'visible','off');
% legend boxoff;
title(['(' figlab(2) ') RMSE'],'FontSize',22);
ylabel('\it{C_x}\rm{(\tau)}','FontSize',22);
xlabel('\it{f}\rm{_{\tau}}','FontSize',22); xlim([min(f_taus) max(f_taus)]);
% xlim([0 max(f_taus)+0.01]);
ylim([yminn_ref ymaxx_ref]);

set(gca,'LineWidth',1);
set(gca,'XTick',0:0.1:0.5);
set(gca,'XTickLabel',{'0.0','0.1','0.2','0.3','0.4','0.5'});
set(gca,'XDir','reverse');

% ax=gca; ax.LineWidth=2;

set(gcf, 'Position', [1 1 1000 500]);
set(gcf, 'Color', 'w');
if esporta=='y'
    saveas(gcf, 'test.png');
    export_fig test.png -png -r600
end






