%% Multiscale Entropy Gaussian - test for linear VAR processes
%%% TEST theoretical values at varying parameter - figure for paper
%%% analysis with fixed steps of f_tau
clear; clc; close all;

%% Parameters
s_vect   = [1 8 4 7  3 5 1 8  2 7  3  1 1 3  1  1];
tau_vect = [1 9 5 10 5 9 2 18 5 20 10 4 5 20 10 20];
f_taus=s_vect./(2*tau_vect);

ncoeff=48; % if FIR, set the number of coeff

%%% AR starting parameters 
ro0=0; %r0=0.65 (fr0=0)
Su=1; 

numconfig=4;
% Configuration 1: vary rho of a single pole, frequency 0.1
conf.rho{1}=[0 0]; conf.freq{1}=[0.1 0.25];
conf.par{1}='ro1'; conf.parval{1}=[0 0.5 0.8 0.9]; %conf.parval{1}=[0 0.3 0.6 0.8 0.9];
% Configuration 2: vary freq of a single pole, rho 0.8
conf.rho{2}=[0.8 0]; conf.freq{2}=[0 0.25];
conf.par{2}='fr1'; conf.parval{2}=[0.1 0.2 0.3 0.4]; %conf.parval{2}=[0.1 0.15 0.25 0.35 0.4];
% Configuration 3: vary rho of a HF pole with fixed LF pole
conf.rho{3}=[0.8 0]; conf.freq{3}=[0.1 0.25];
conf.par{3}='ro2'; conf.parval{3}=[0 0.3 0.6 0.8]; %conf.parval{3}=[0 0.3 0.6 0.8 0.9];
% Configuration 4: vary freq of a HF pole with fixed LF pole
conf.rho{4}=[0.8 0.8]; conf.freq{4}=[0.1 0];
conf.par{4}='fr2'; conf.parval{4}=[0.15 0.2 0.25 0.3]; %conf.parval{4}=[0.15 0.2 0.25 0.3 0.35];

esporta='n';

%% MSE Analysis
nscales=length(f_taus);
% MSEavg=nan*ones(nscales,numcond); Eavg=MSEavg;
% MSEdws=nan*ones(nscales,numcond); Edws=MSEdws;
% MSEdws_n=nan*ones(nscales,numcond); Edws_n=MSEdws_n;
for iconfig=1:numconfig
    ro1=conf.rho{iconfig}(1); fr1=conf.freq{iconfig}(1);
    ro2=conf.rho{iconfig}(2); fr2=conf.freq{iconfig}(2);
    
    numcond=length(conf.parval{iconfig}); %number of conditions
    for cnt=1:numcond
        eval(strcat([conf.par{iconfig} '=conf.parval{iconfig}(' num2str(cnt) ');'])); %set parameter for the given condition
        clc; disp(['conf. ' int2str(iconfig) ', ' conf.par{iconfig} '=' num2str(conf.parval{iconfig}(cnt))]);

        for is=1:nscales
            s=round(s_vect(is));
            tau=round(tau_vect(is));

            % generates AR process
            z0=ro0;
            z1a=ro1*(cos(2*pi*fr1)+1i*sin(2*pi*fr1)); z1b=ro1*(cos(2*pi*fr1)-1i*sin(2*pi*fr1));
            z2a=ro2*(cos(2*pi*fr2)+1i*sin(2*pi*fr2)); z2b=ro2*(cos(2*pi*fr2)-1i*sin(2*pi*fr2));
            z=[z0; z1a; z1b; z2a; z2b];
            Apol=poly(z);
            Apol(Apol==0)=[];
            Am=-Apol(2:end);
            if isempty(Am) % to handle the case of white noise (AR process of order 0)
                Am=0;
            end 
            p=length(Am);
            E=eye(p);AA=[Am;E(1:end-1,:)];lambda=eig(AA);lambdamax=max(abs(lambda));
            if lambdamax>=1% stability check
                error('The simulated VAR process is not stable');
            end;

            % theoretical MSE estimate
            [tmp1, tmp2, tmp3, tmp4, tmp5]=ar2mse(Am,Su,tau,s,ncoeff);
            MSEdws(is,cnt,iconfig)=tmp1;
            MSEdws_n(is,cnt,iconfig)=tmp2;
            
            %Edws(is,cnt,iconfig)=tmp4; MSEavg(is,cnt,iconfig)=tmp3; Eavg(is,cnt,iconfig)=tmp5;
        end
    end
end


%% figs
set(0,'defaultAxesFontSize', 18);
set(0,'defaultTextFontSize', 18);
set(0,'defaultTextFontName', 'Times');
set(0,'defaultAxesFontName', 'Times');
conf.lab{1}='\rho_1';
conf.lab{2}='\it{f}\rm{_1}';
conf.lab{3}='\rho_2';
conf.lab{4}='\it{f}\rm{_1}';
stringa{1}='(a) theoretical MSE';
stringa{2}='(b) theoretical MSE';
stringa{3}='(c) theoretical MSE';
stringa{4}='(d) theoretical MSE';
yminn=[0.25 0.5 0.5 -0.5];
yticc{1}=[0.25, 0.5, 0.75, 1, 1.25, 1.5];
yticc{2}=[0.5, 0.75, 1, 1.25, 1.5];
yticc{3}=[0.5 0.75, 1, 1.25, 1.5];
yticc{4}=[-0.5, 0, 0.5, 1, 1.5];
%col0=[146 54 135]/255; %purple
col1=[128 128 128]/255; %gray
col2=[255 128 0]/255; %orange
col3=[64 128 0]/255; %green
col4=[0 128 255]/255; % lighblue
colori=[col1; col2; col3; col4];
figure(1); clf
for iconfig=1:numconfig
    clear labella;
    for cnt=1: length(conf.parval{iconfig})
        subplot(2,2,iconfig)
        plot(f_taus,MSEdws_n(:,cnt,iconfig),'marker','o','markerfacecolor',colori(cnt,:),'Color',colori(cnt,:),'LineWidth',1.5);hold on;
        labella{cnt}=[conf.lab{iconfig} '=' num2str(conf.parval{iconfig}(cnt))];
    end
    hleg=legend(labella,'location','SouthEast');
    set(hleg,'FontSize',16); %set(hleg,'visible','off');
    legend boxoff;
    title(stringa{iconfig});
    ylabel('\it{C_x}\rm{(\tau)}','FontSize',22);
    xlabel('\it{f}\rm{_{\tau}}','FontSize',22);
    xlim([0 max(f_taus)+0.01]);
    ylim([yminn(iconfig) 1.5]);%tmp=ylim; ylim([tmp(1) 1.5]);
    set(gca,'XDir','reverse');
    set(gca,'YTick',yticc{iconfig});% set(gca,'XTickLabel',{'0','10','30','50','60'});
    set(gca,'XTick',0:0.1:0.5);
    set(gca,'XTickLabel',{'0','0.1','0.2','0.3','0.4','0.5'});
end


set(gcf, 'Color', 'w');
% use toolbox to export: fantastic! 
if esporta=='y'
    set(gcf, 'Position', [1 1 800 800]);
    %saveas(gcf, 'test.png');
    %export_fig test2.png -m2.5
    export_fig test.png -png -r600
end

